/***************************************************************************
 *   Copyright (C) 2003 by Norman Jordan                                   *
 *   njordan@shaw.ca                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

 /***************************************************************************
 *   Portions of code were take from wmnetselect Copyright (C) 2000        *
 *   Patrick Hill apathos@bham.net                                         *
 ***************************************************************************/

#include <stdio.h>

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include <gdk/gdk.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xmu/WinUtil.h>

#include <gdk/gdkx.h>

#include "gnome-netselect.h"
#include "browser_utils.h"

#define MOZILLA_VERSION_PROP	"_MOZILLA_VERSION"
#define MOZILLA_LOCK_PROP	"_MOZILLA_LOCK"
#define MOZILLA_COMMAND_PROP	"_MOZILLA_COMMAND"
#define MOZILLA_RESPONSE_PROP	"_MOZILLA_RESPONSE"
#define MOZILLA_URL_PROP	"_MOZILLA_URL"
#define DEFAULT_TIMEOUT 10000

static Atom XA_MOZILLA_VERSION  = 0;
static Atom XA_MOZILLA_LOCK     = 0;
static Atom XA_MOZILLA_COMMAND  = 0;
static Atom XA_MOZILLA_RESPONSE = 0;

static long Length = (65536 / sizeof(long));

query_target_type query_targets[] = {
  { "google",
    "http://www.google.com/search?q=",
    "&num=30"
  },
  { "altavista",
    "http://www.altavista.com/cgi-bin/query?pg=q&text=yes&what=web&fnt=d&q=",
    ""
  },
  { "excite",
    "http://search.excite.com/search.gw?search=",
    "&start=0&showSummary=true&perPage=30"
  },
  { "hotbot",
    "http://hotbot.lycos.com/?MT=",
    "",
  },
  { "alltheweb",
    "http://alltheweb.com/search?type=all&cat=web&query=",
    ""
  },
  { "yahoo",
    "http://search.yahoo.com/search?sub=Search&fr=fp-top&p=",
    ""
  },
};
gint num_search_engines = 6;

Window
FindMozWin()
{
  Window win = None;
  Display *dpy = gdk_x11_get_default_xdisplay();

  XSetErrorHandler(ErrorHandler);
  if(moz_win != None)
  {
    if(Check4Moz(dpy, moz_win))
    {
      win = moz_win;
    } else
    {
      moz_win = None;
    }
  }
  if(win == None)
  {
    if((win = SearchForMozWindow(dpy)) == None)
    {
      return None;
    }
    moz_win = win;
  }
  return win;
}

Window
SearchForMozWindow(Display *dpy)
{
  int i;
  Window root = RootWindowOfScreen(DefaultScreenOfDisplay(dpy));
  Window root2, parent, *kids;
  unsigned int nkids;
  Window result = None;

  if(!XQueryTree(dpy, root, &root2, &parent, &kids, &nkids))
  {
    return None;
  }
  if(root != root2) {
    return None;
  }
  if(parent != None) {
    return None;
  }
  if(!(kids && nkids))
  {
    return None;
  }

  for(i = 0; i < nkids; i++)
  {
    Window w = XmuClientWindow(dpy, kids[i]);
    if(Check4Moz(dpy, w))
    {
      result = w;
      break;
    }
  }
  return result;
}

int
Check4Moz(Display *dpy, Window win)
{
  Atom type;
  int format;
  unsigned long nitems, bytes_after;
  unsigned char *version = NULL;
  int status = XGetWindowProperty(dpy, win, XA_MOZILLA_VERSION, 0,
                                  Length, False, XA_STRING, &type,
                                  &format, &nitems, &bytes_after, &version);

  if(status != Success || !version)
  {
  if(version)
    XFree(version);
    return 0;
  }
  XFree(version);
  return 1;
}

int
GetLock(Display *dpy, Window win)
{
  char lock_data[255];
  Bool locked = False;

  sprintf(lock_data, "pid%d@", getpid());
  if(gethostname(lock_data + strlen(lock_data), 100) == -1)
  {
    fprintf (stderr, "gethostname() error\n");
    return 0;
  }

  do
  {
    int result;
    Atom type;
    int format;
    unsigned long nitems, bytes_after;
    unsigned char *data = NULL;

    /*
     * Grab the server so nobody else can do anything
     */

    XGrabServer(dpy);

    /*
     * Check for Mozilla Lock
     */
    result = XGetWindowProperty(dpy, win, XA_MOZILLA_LOCK, 0,
                                Length, False, XA_STRING, &type,
                                &format, &nitems, &bytes_after, &data);

    if(result != Success || type == None)
    {
      /*
       * It's not locked now, lock it
       */
      XChangeProperty(dpy, win, XA_MOZILLA_LOCK, XA_STRING,
                      8, PropModeReplace,
                      (unsigned char *)lock_data,
                      strlen(lock_data));
      locked = True;
    }
    /*
     * Release the server grab
     */

    XUngrabServer(dpy);
    XSync(dpy, False);

    if(!locked)
    {
      /*
       * There was already a lock in place.  Wait for
       * a PropertyDelete event.
       */

      while(1)
      {
        XEvent event;
        XNextEvent(dpy, &event);
        if(event.xany.type == DestroyNotify &&
          event.xdestroywindow.window == win)
        {
        /*
         * fprintf (stderr, "window 0x%x unexpectedly
         * destroyed.\n", win);
         */
          return -1;
        } else if(event.xany.type == PropertyNotify &&
                  event.xproperty.state == PropertyDelete &&
                  event.xproperty.window == win &&
                  event.xproperty.atom == XA_MOZILLA_LOCK)
        {
          break;
        }
      }
    }
    if(data)
      XFree(data);
  } while(!locked);

  return locked == True ? 1 : 0;
}

void
ReleaseLock(Display *dpy, Window win)
{
  Atom type;
  int format;
  unsigned long nitems, bytes_after;
  unsigned char *data;

  XGetWindowProperty(dpy, win, XA_MOZILLA_LOCK, 0,
                     Length, True /* delete */,
                     XA_STRING, &type, &format,
                     &nitems, &bytes_after, &data);
  if(data)
    XFree(data);
}

int
SendCommand(Window win, const char *command)
{
  int result, counter = 0;
  Bool done = False;
  Atom type;
  Display *dpy = gdk_x11_get_default_xdisplay();
  int format;
  unsigned long nitems, bytes_after;
  unsigned char *data = 0;

  /*
   * Select for PropertyChange events on the Mozilla window
   */

  XSelectInput(dpy, win, (PropertyChangeMask | StructureNotifyMask));

  if(GetLock(dpy, win) == 0)
  {
    XSelectInput(dpy, win, 0);
    return 1;
  }

  /*
   * We've got a successful lock, so send the command to Mozilla
   */

  /*
   * fprintf (stderr, "(writing " MOZILLA_COMMAND_PROP " \"%s\" to 0x%x)\n",
   *  command, (unsigned int) win);
   */

  XChangeProperty(dpy, win, XA_MOZILLA_COMMAND,
                  XA_STRING, 8, PropModeReplace,
                  (unsigned char *)command, strlen(command));

  while(!done)
  {
    XEvent event;
    XNextEvent (dpy, &event);
    if(event.xany.type == DestroyNotify &&
       event.xdestroywindow.window == win)
    {
      /*
       * fprintf (stderr, "window 0x%x was destroyed.\n",
       * (unsigned int) win);
       */
      return -1;
    } else if(event.xany.type == PropertyNotify &&
              event.xproperty.state == PropertyNewValue &&
              event.xproperty.window == win &&
              event.xproperty.atom == XA_MOZILLA_RESPONSE)
    {

      result = XGetWindowProperty(dpy, win,
                                  XA_MOZILLA_RESPONSE, 0,
                                  Length, True /* delete */,
                                  XA_STRING, &type, &format,
                                  &nitems, &bytes_after, &data);

      if(result != Success)
      {
        /*
         * fprintf (stderr, "Failed to read response from Mozilla\n");
         */
        ReleaseLock(dpy, win);
        XSelectInput(dpy, win, 0);
        return 1;
      }
      /* if(DEBUG && result == Success && data && *data)
      {
        fprintf(stderr, "(server sent " MOZILLA_RESPONSE_PROP
                " \"%s\" to 0x%x.)\n", data, (unsigned int)win);
      } */

      /*
       * counter gets us out if we are stuck waiting for a lock.
       */
      if(!data || (++counter > 9))
      {
        /*
         * printf ("No data returned from Mozilla\n");
         */
        ReleaseLock(dpy, win);
        XSelectInput(dpy, win, 0);
        return 1;
      }
      if(data)
      {
        XFree(data);
        done = True;
      }
        ReleaseLock(dpy, win);
        XSelectInput(dpy, win, 0);
      }
    }
    return 0;
}

int ErrorHandler
(Display *dpy, XErrorEvent *event)
{
  return 0;
}

void InitAtoms() {
  Display *dpy = gdk_x11_get_default_xdisplay();

  if(!XA_MOZILLA_VERSION)
    XA_MOZILLA_VERSION = XInternAtom (dpy, MOZILLA_VERSION_PROP, False);
  if(!XA_MOZILLA_LOCK)
    XA_MOZILLA_LOCK = XInternAtom (dpy, MOZILLA_LOCK_PROP, False);
  if(!XA_MOZILLA_COMMAND)
    XA_MOZILLA_COMMAND = XInternAtom (dpy, MOZILLA_COMMAND_PROP, False);
  if(!XA_MOZILLA_RESPONSE)
    XA_MOZILLA_RESPONSE = XInternAtom (dpy, MOZILLA_RESPONSE_PROP, False);
}

int
URLSelection(char *prefix, char *select, int NewWin, int raise)
{
  char *netcommand = NULL, *NewWinString = "";
  int error;
  Window win;

  win = FindMozWin();

  if(win == None)
  {
    /* No Mozilla window: start one.  */
    /* prefix + & + "" + spaces + 2>/dev/null = 24 */
    netcommand = (char *)malloc(sizeof(char) * (strlen(browser_path) + strlen(select) + 25));
    if(netcommand == NULL)
    {
      fprintf(stderr, "Could not allocate memory\n");
      return -1;
    }
    sprintf(netcommand, "%s \"%s%s\" 2>/dev/null &", browser_path,
            prefix, select);
    error = system(netcommand);
  } else /* win != None */
  {
    /* netcommand = (char *) malloc(sizeof(char) * (strlen(select) + 27));
     */
    netcommand = (char *) malloc(sizeof(char) * (strlen(select) + 35));
    if(netcommand == NULL)
    {
      fprintf (stderr, "Could not allocate memory\n");
      return -1;
    }
    if(NewWin == 1) NewWinString = ",new-window";
    else if(NewWin == 2) NewWinString = ",new-tab";
    sprintf(netcommand, "openURL(%s%s%s%s)",
            prefix, select, NewWinString,
    raise ? ",noraise" : "");

    error = SendCommand(win, netcommand);
  }
  /* printf ("\nerr = <%d>, netcommand=<%s> ",error, netcommand); */
  if(netcommand)
    free(netcommand);
  return error;
}

int
Search4Selection(char *select, int NewWin, int raise)
{
  query_target_type target;
  char *netcommand = NULL, *NewWinString = "";
  int error;
  Window win;

  if(search_engine < 0 || search_engine >= num_search_engines)
  {
    return -1;
  }
  target = query_targets[search_engine];

  win = FindMozWin();

  if(win == None)
  {
    /* No Mozilla window, start one.  */
    /* 2>/dev/null + spaces + & + "" = 17 */
    netcommand = (char *)malloc(sizeof(char) * (strlen(browser_path) + strlen(target.pre) +
                                strlen(select) + strlen(target.post) + 18));
    if(netcommand == NULL)
    {
      fprintf(stderr, "Could not allocate memory\n");
      return -1;
    }
    sprintf(netcommand, "%s \"%s%s%s\" 2>/dev/null &", browser_path,
            target.pre, select, target.post);
    error = system (netcommand);
  } else /* win != None */ {
    /* strlen ("openURL(,new-window)") = 20 */
    netcommand = (char *)
    malloc(sizeof(char) * (strlen(target.pre) + strlen(select) + strlen(target.post) + 28));
    if(netcommand == NULL)
    {
      fprintf (stderr, "Could not allocate memory\n");
      return -1;
    }
    if(NewWin == 1) NewWinString = ",new-window";
    else if(NewWin == 2) NewWinString = ",new-tab";
    sprintf(netcommand, "openURL(%s%s%s%s%s)",
            target.pre, select, target.post, NewWinString,
            raise ? ",noraise" : "");

    error = SendCommand(win, netcommand);
  }
  if(netcommand) free(netcommand);
  return error;
}
