/***************************************************************************
 *   Copyright (C) 2003 by Norman Jordan                                   *
 *   njordan@shaw.ca                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

/***************************************************************************
 *   Portions of code were take from wmnetselect Copyright (C) 2000        *
 *   Patrick Hill apathos@bham.net                                         *
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <glib/gprintf.h>

#include "selection_utils.h"

void
killstring(gchar *str, gchar *killstring)
{
  GString *new_string;
  int i, j, kill_length;

  /* Just leave if there aren't any newlines to mess with. */
  if(str == NULL || killstring == NULL ||
     g_strrstr(str, killstring) == NULL)
  {
    return;
  }

  new_string = g_string_new(killstring);
  if((kill_length = strlen((char *)killstring)) == 0)
    return;

  new_string = g_string_new(str);

  for(i = 0;i < new_string->len;i++) {
    gboolean match = FALSE;
    if(new_string->str[i] == killstring[0])
    {
      for(j = i + 1;j < new_string->len;j++)
      {
        if(j - i == kill_length)
        {
          match = TRUE;
          break;
        } else if(new_string->str[j] == killstring[j - i])
        {
          break;
        }
      }
    }

    if(match == TRUE)
      g_string_erase(new_string, i, kill_length);
  }

  g_stpcpy(str, new_string->str);

  g_string_free(new_string, TRUE);
}


gchar *
encode(gchar *url, int button)
{
  gchar esc[4], *uns, *result;
  GString *ncoded;
  int i, j, len;
  static gchar *unsafe = "!*'(),{};|\\^~[]`";
  static gchar *punctuation = "#?&+$%\":<>/.=!*'(),{};|\\^~[]`";

  if(url == NULL || strlen((char *)url) == 0)
  {
    result = (gchar *)g_malloc(sizeof(gchar));
    *result = '\0';
    return(result);
  }

  ncoded = g_string_new(url);

  if(ncoded == NULL) {
    fprintf (stderr, "Could not allocate memory\n");
    return NULL;
  }

  if(button == 3)
    uns = punctuation;
  else
    uns = unsafe;
  len = strlen((char *)uns);
  i=j=0;

  while(i < ncoded->len)
  {
    gboolean match = FALSE;
    for(j = 0;j < len;j++) {
      if(ncoded->str[i] == uns[j])
      {
        match = TRUE;
        break;
      }
    }

    if(match == TRUE)
    {
      g_sprintf(esc, "%%%2x", ncoded->str[i]);
      esc[2] = g_ascii_toupper(esc[2]);
      g_string_erase(ncoded, i, 1);
      g_string_insert_len(ncoded, i, esc, 4);

      i += 3;
    } else if(g_ascii_isspace(ncoded->str[i]))
    {
      ncoded->str[i] = '+';
      i++;
    } else
    {
      i++;
    }
  }

  result = (gchar *)g_malloc(sizeof(gchar) * ncoded->len + 15);
  g_stpcpy(result, ncoded->str);

  g_string_free(ncoded, TRUE);

  return(result);
}


char *
protocol(gchar *url, int noproto) {
  gchar *ptcol;
  gchar *ptr;

  ptcol = (gchar *)g_malloc(sizeof(gchar) * 8);

  if(noproto || g_strrstr(url, ":/"))
    return(strcpy(ptcol,""));

  /* Are the first 4 chars "ftp." ?  */
  ptr = url;
  if(g_strstr_len(url, 4, "ftp.") == ptr)
    return(strcpy(ptcol,"ftp://"));

  /* contains a '@' but no '/'  */
  if(g_strrstr(url, "/") == NULL && g_strrstr(url,"@") != NULL)
    return(strcpy(ptcol, "mailto:"));

  /* first character is a '/', then it must be a file */
  if(url[0] == '/')
    return(strcpy(ptcol, "file:"));

  return(strcpy(ptcol, "http://"));
}
